/*
 *   Copyright  1995-2001, AROS. All Rights Reserved.
 *   Copyright  2001-2010, MorphOS Team. All Rights Reserved.
 *
 *   $Id: malti.c,v 1.1 2010/11/27 11:55:17 tokai Exp $
 *
 *   Desc: malti.country description file.
 *   Lang: Maltese
 *   Char: ISO 8859-3
 */

#include <exec/types.h>
#include <aros/system.h>
#include <exec/resident.h>
#include <exec/libraries.h>
#include <libraries/locale.h>
#include <dos/dos.h>

#include <proto/exec.h>
#include <aros/libcall.h>

#include <aros/debug.h>

#define LANGSTR "malti"     /* String version of above     */
#define LANGVER 51          /* Version number of language  */
#define LANGREV 1           /* Revision number of language */

#define LANGTAG "\0$VER: "LANGSTR".language 51.1 (23.11.2010)  1995-2001 AROS,  2001-2010 MorphOS Team"



/* ----------------------------------------------------------------------- */

/*  Bit masks for locale .language functions. Only implement GetString() 
 */
#define LF_GetLangStr       (1L << 3)
#define LF_GetLangStrU      (1L << 17)

#define LF_MASK             (LF_GetLangStr | LF_GetLangStrU)


/*  Arrays for Maltese character type/conversion 
 */
extern const STRPTR        __malti_strings[];
extern const CONST_WSTRPTR __malti_stringsu[];




/*  -------------------------------------------------------------------------
 *  Library definition, you should not need to change any of this.
 *  -------------------------------------------------------------------------
 */

struct Language
{
    struct Library library;
    struct ExecBase *sysbase;
    BPTR seglist;
};

extern const UBYTE name[];
extern const UBYTE version[];
extern const APTR inittabl[4];
extern void *const functable[];
extern struct Language *AROS_SLIB_ENTRY(init,language)();
extern struct Language *AROS_SLIB_ENTRY(open,language)();
extern BPTR AROS_SLIB_ENTRY(close,language)();
extern BPTR AROS_SLIB_ENTRY(expunge,language)();
extern int AROS_SLIB_ENTRY(null,language)();
extern ULONG AROS_SLIB_ENTRY(mask,language)();
extern const char end;

int entry(void)
{
	return -1;
}

const struct Resident languageTag =
{
	RTC_MATCHWORD,
	(struct Resident *)&languageTag,
	(APTR)&languageTag + 1,
	RTF_PPC | RTF_AUTOINIT,
	LANGVER,
	NT_LIBRARY,
	-120,
	(STRPTR)name,
	(STRPTR)&version[7],
	(ULONG *)inittabl
};

const UBYTE name[]    = LANGSTR ".language";
const UBYTE version[] = LANGTAG;

const APTR inittabl[4] =
{
	(APTR)sizeof(struct Language),
	(APTR)functable,
	(APTR)NULL,
	&AROS_SLIB_ENTRY(init,language)
};

struct ExecBase *mySysBase;

#ifdef __MORPHOS__
struct Language *LIB_init(struct Language *language, BPTR segList, struct ExecBase *SysBase)
#else
AROS_LH2(struct Language *, init,
    AROS_LHA(struct Language *, language, D0),
    AROS_LHA(BPTR,             segList, A0),
    struct ExecBase *, SysBase, 0, language)
#endif
{
	AROS_LIBFUNC_INIT

	/*  You could just as easily do this bit as the InitResident()
	 *  datatable, however this works just as well.
	 */
	language->library.lib_Node.ln_Type                = NT_LIBRARY;
	language->library.lib_Node.ln_Pri                 = -120;
	(const UBYTE *)language->library.lib_Node.ln_Name = name;
	language->library.lib_Flags                       = LIBF_SUMUSED | LIBF_CHANGED;
	language->library.lib_Version                     = LANGVER;
	language->library.lib_Revision                    = LANGREV;
	(const UBYTE *)language->library.lib_IdString     = &version[7];

	language->seglist = segList;
	language->sysbase = SysBase;
	mySysBase         = SysBase;

	/*  Although it is unlikely, you would return NULL if you for some
	 *  unknown reason couldn't open.
	 */	
	return language;
}

#define SysBase language->sysbase

AROS_LH1(struct Language *, open,
	AROS_LHA(ULONG, version, D0), 
	struct Language *, language, 1, language)
{
	AROS_LIBFUNC_INIT

	language->library.lib_OpenCnt++;
	language->library.lib_Flags &= ~LIBF_DELEXP;

	/* Again return NULL if you could not open 
	 */
	return language;

	AROS_LIBFUNC_EXIT
}

AROS_LH0(BPTR, close, struct Language *, language, 2, language)
{
	AROS_LIBFUNC_INIT

	if(! --language->library.lib_OpenCnt)
	{
		/*  Delayed expunge pending? 
		 */
		if(language->library.lib_Flags & LIBF_DELEXP)
		{
			/* Yes, expunge the library
			 */
			return AROS_LC0(BPTR, expunge, struct Language *, language, 3, language);
		}
	}

	return NULL;

	AROS_LIBFUNC_EXIT
}


AROS_LH0(BPTR, expunge, struct Language *, language, 3, language)
{
	AROS_LIBFUNC_INIT

	BPTR ret;

	if(language->library.lib_OpenCnt)
	{
		/* Can't expunge, we are still open
		 */
		language->library.lib_Flags |= LIBF_DELEXP;
		return 0;
	}

	REMOVE(&language->library.lib_Node);
	ret = language->seglist;

	FreeMem((UBYTE *)language - language->library.lib_NegSize,
		language->library.lib_PosSize + language->library.lib_NegSize);

	return ret;

	AROS_LIBFUNC_EXIT
}


AROS_LH0I(int, null, struct Language *, language, 4, language)
{
	AROS_LIBFUNC_INIT

	return 0;

	AROS_LIBFUNC_EXIT
}




/*  ------------------------------------------------------------------------
 *  Language specific functions
 *  ------------------------------------------------------------------------
 */

#undef SysBase
#define SysBase	mySysBase

/*  ULONG LanguageMask():
 *
 *  This function is to inform locale.library what functions it should
 *  use from this library. This is done by returning a bitmask containing
 *  1's for functions to use, and 0's for functions to ignore.
 *
 *  Unused bits MUST be 0 for future compatibility.
 */
AROS_LH0(ULONG, mask, struct Language *, language, 5, language)
{
	AROS_LIBFUNC_INIT

	return ( LF_MASK );

	AROS_LIBFUNC_EXIT
}

/*  STRPTR GetLangString(ULONG num): Language function 3
 *
 *  This function is called by GetLocaleStr() and should return
 *  the string matching the string id passed in as num.
 */
AROS_LH1(STRPTR, getlangstring,
    AROS_LHA(ULONG, id, D0),
    struct LocaleBase *, LocaleBase, 9, language)
{
	AROS_LIBFUNC_INIT

	if(id < MAXSTRMSG)
	{
		return __malti_strings[id];
	}
	else
	{
		return NULL;
	}

	AROS_LIBFUNC_EXIT
}

CONST_WSTRPTR LIB_getlangstringu(ULONG id, struct LocaleBase *LocaleBase)
{
	if (id < MAXSTRMSG)
	{
		return __malti_stringsu[id];
	}
	else
	{
		return NULL;
	}
}


/*  -----------------------------------------------------------------------
 *  Library function table - you will need to alter this
 *  I have this right here at the end of the library so that I do not
 *  have to have prototypes for the functions. Although you could do that.
 *  -----------------------------------------------------------------------
 */
void *const functable[] =
{
	(void *const) FUNCARRAY_BEGIN,
	(void *)FUNCARRAY_32BIT_NATIVE,
	&AROS_SLIB_ENTRY(open,language),
	&AROS_SLIB_ENTRY(close,language),
	&AROS_SLIB_ENTRY(expunge,language),
	&AROS_SLIB_ENTRY(null,language),
	&AROS_SLIB_ENTRY(mask,language),

	/* Note, shorter function table, as only getlangstring is used
	 */

	/* 0 - 3 */
	&AROS_SLIB_ENTRY(null, language),
	&AROS_SLIB_ENTRY(null, language),
	&AROS_SLIB_ENTRY(null, language),    
	&AROS_SLIB_ENTRY(getlangstring, language),

	/* 4 - 16 */
	&AROS_SLIB_ENTRY(null, language),
	&AROS_SLIB_ENTRY(null, language),
	&AROS_SLIB_ENTRY(null, language),
	&AROS_SLIB_ENTRY(null, language),
	&AROS_SLIB_ENTRY(null, language),
	&AROS_SLIB_ENTRY(null, language),
	&AROS_SLIB_ENTRY(null, language),
	&AROS_SLIB_ENTRY(null, language),
	&AROS_SLIB_ENTRY(null, language),
	&AROS_SLIB_ENTRY(null, language),
	&AROS_SLIB_ENTRY(null, language),
	&AROS_SLIB_ENTRY(null, language),
	&AROS_SLIB_ENTRY(null, language),
	(void *)-1,

	/* 17 */
	(void *) FUNCARRAY_32BIT_SYSTEMV,
	&LIB_getlangstringu,
	(void *)-1L,

	(void *const) FUNCARRAY_END
};


/*  Note how only the required data structures are kept...
 *
 *  This is the list of strings. It is an array of pointers to strings,
 *  although how it is laid out is implementation dependant.
 */
static const STRPTR __malti_strings[] =
{
	/* A blank string
	 */
	"",

	/*  The days of the week. Starts with the first day of the week.
	 *
	 *  In English this would be Sunday, this depends upon the settings
	 *  of Locale->CalendarType.
	 *
	 *  #warning: stegerg: I think this must always start with Sunday and not what comment above says
	 */
	"il-add",
	"it-Tnejn",
	"it-Tlieta",
	"l-Erbga",
	"il-amis",
	"il-imga",
	"is-Sibt",

	/*  Abbreviated days of the week
	 */
	"ad", "Tne", "Tli", "Erb", "am", "im", "Sib",

	/*  Months of the year 
	 */
	"Jannar",  "Frar",     "Marzu",
	"April",   "Mejju",    "unju",
	"Lulju",   "Awissu",   "Settembru",
	"Ottubru", "Novembru", "Diembru",

	/*  Abbreviated months of the year
	 */
	"Jan", "Fra", "Mar",
	"Apr", "Mej", "un",
	"Lul", "Awi", "Set",
	"Ott", "Nov", "Di",

	/*  Responses
	 */
	"Iva", /* Yes, affirmative response */
	"Le",  /* No,  negative response    */

	/*  AM/PM strings
	 *
	 *  AM 0000 -> 1159,
	 *  PM 1200 -> 2359
	 */
	"AM", "PM",

	/*  Soft and hard hyphens
	 */
	"", "-",

	/*  Open and close quotes
	 */
	"\"", "\"",

	/*  Days: But not actual day names
	 *
	 *  Yesterday - the day before the current
	 *  Today     - the current day
	 *  Tomorrow  - the next day
	 *  Future.
	 */
	"Ilbiera",
	"Illum",
	"Gada",
	"Fil-futur"
};

static const CONST_WSTRPTR __malti_stringsu[] =
{
	/* A blank string
	 */
	(CONST_WSTRPTR)L"",

	/*  The days of the week. Starts with the first day of the week.
	 *
	 *  In English this would be Sunday, this depends upon the settings
	 *  of Locale->CalendarType.
	 *
	 *  #warning: stegerg: I think this must always start with Sunday and not what comment above says
	 */
	(CONST_WSTRPTR)L"il-Ħadd",
	(CONST_WSTRPTR)L"it-Tnejn",
	(CONST_WSTRPTR)L"it-Tlieta",
	(CONST_WSTRPTR)L"l-Erbgħa",
	(CONST_WSTRPTR)L"il-Ħamis",
	(CONST_WSTRPTR)L"il-Ġimgħa",
	(CONST_WSTRPTR)L"is-Sibt",

	/*  Abbreviated days of the week
	 */
	(CONST_WSTRPTR)L"Ħad", (CONST_WSTRPTR)L"Tne", (CONST_WSTRPTR)L"Tli", (CONST_WSTRPTR)L"Erb", (CONST_WSTRPTR)L"Ħam", (CONST_WSTRPTR)L"Ġim", (CONST_WSTRPTR)L"Sib",

	/*  Months of the year 
	 */
	(CONST_WSTRPTR)L"Jannar",  (CONST_WSTRPTR)L"Frar",     (CONST_WSTRPTR)L"Marzu",
	(CONST_WSTRPTR)L"April",   (CONST_WSTRPTR)L"Mejju",    (CONST_WSTRPTR)L"Ġunju",
	(CONST_WSTRPTR)L"Lulju",   (CONST_WSTRPTR)L"Awissu",   (CONST_WSTRPTR)L"Settembru",
	(CONST_WSTRPTR)L"Ottubru", (CONST_WSTRPTR)L"Novembru", (CONST_WSTRPTR)L"Diċembru",

	/*  Abbreviated months of the year
	 */
	(CONST_WSTRPTR)L"Jan", (CONST_WSTRPTR)L"Fra", (CONST_WSTRPTR)L"Mar",
	(CONST_WSTRPTR)L"Apr", (CONST_WSTRPTR)L"Mej", (CONST_WSTRPTR)L"Ġun",
	(CONST_WSTRPTR)L"Lul", (CONST_WSTRPTR)L"Awi", (CONST_WSTRPTR)L"Set",
	(CONST_WSTRPTR)L"Ott", (CONST_WSTRPTR)L"Nov", (CONST_WSTRPTR)L"Diċ",

	/*  Responses
	 */
	(CONST_WSTRPTR)L"Iva", /* Yes, affirmative response */
	(CONST_WSTRPTR)L"Le",  /* No,  negative response    */

	/*  AM/PM strings
	 *
	 *  AM 0000 -> 1159,
	 *  PM 1200 -> 2359
	 */
	(CONST_WSTRPTR)L"AM", (CONST_WSTRPTR)L"PM",

	/*  Soft and hard hyphens
	 */
	(CONST_WSTRPTR)L"", (CONST_WSTRPTR)L"-",

	/*  Open and close quotes
	 */
	(CONST_WSTRPTR)L"\"", (CONST_WSTRPTR)L"\"",

	/*  Days: But not actual day names
	 *
	 *  Yesterday - the day before the current
	 *  Today     - the current day
	 *  Tomorrow  - the next day
	 *  Future.
	 */
	(CONST_WSTRPTR)L"Ilbieraħ",
	(CONST_WSTRPTR)L"Illum",
	(CONST_WSTRPTR)L"Għada",
	(CONST_WSTRPTR)L"Fil-futur"
};
